package com.example.wfwebapi.controller;

import java.util.List;
import com.example.wfwebapi.exception.ResourceNotFoundException;
import com.example.wfwebapi.model.Favorite;
import com.example.wfwebapi.model.FavoriteId;
import com.example.wfwebapi.model.User;
import com.example.wfwebapi.model.Quote;
import com.example.wfwebapi.repository.FavoriteRepository;
import com.example.wfwebapi.repository.UserRepository;
import com.example.wfwebapi.repository.QuoteRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping("/api/v1/favorite")
public class FavoriteController {

    @Autowired
    private FavoriteRepository favoriteRepository;

    @Autowired
    private UserRepository userRepository;

    @Autowired
    private QuoteRepository quoteRepository;

    @GetMapping("/{userId}")
    public List<Favorite> getFavoritesByUser(@PathVariable Long userId) {
        return favoriteRepository.findByUser_Id(userId);
    }

    @PostMapping
    public Favorite addFavorite(@RequestParam Long user, @RequestParam Long quote) {
        User u = userRepository.findById(user)
                .orElseThrow(() -> new ResourceNotFoundException("Utilisateur non trouvé : " + user));
        Quote q = quoteRepository.findById(quote)
                .orElseThrow(() -> new ResourceNotFoundException("Citation non trouvée : " + quote));

        Favorite favorite = new Favorite();
        favorite.setId(new FavoriteId(user, quote));
        favorite.setUser(u);
        favorite.setQuote(q);
        return favoriteRepository.save(favorite);
    }

    @DeleteMapping
    public void deleteFavorite(@RequestParam Long user, @RequestParam Long quote) {
        FavoriteId id = new FavoriteId(user, quote);
        if (!favoriteRepository.existsById(id))
            throw new ResourceNotFoundException("Favorite non trouvée");
        favoriteRepository.deleteById(id);
    }
}
