package com.example.wfwebapi.assembler;

import com.example.wfwebapi.controller.QuoteController;
import com.example.wfwebapi.model.Quote;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.hateoas.EntityModel;

import static org.junit.jupiter.api.Assertions.*;
import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

class QuoteModelAssemblerTest {

    private QuoteModelAssembler assembler;
    private Quote quote;
    private EntityModel<Quote> model;

    @BeforeEach
    void setUp() {
        assembler = new QuoteModelAssembler();
        quote = new Quote();
        quote.setId(1L);
        model = assembler.toModel(quote);
    }

    @Test
    void testModelContent() {
        assertEquals(quote, model.getContent());
    }

    @Test
    void testHasSelfLink() {
        assertTrue(model.getLinks().hasLink("self"));
    }

    @Test
    void testHasQuotesLink() {
        assertTrue(model.getLinks().hasLink("quotes"));
    }

    @Test
    void testSelfLinkHref() {
        String expectedHref = linkTo(methodOn(QuoteController.class).getQuoteById(1L)).withSelfRel().getHref();
        String actualHref = model.getRequiredLink("self").getHref();
        assertEquals(expectedHref, actualHref);
    }

    @Test
    void testQuotesLinkHref() {
        String expectedHref = linkTo(methodOn(QuoteController.class).getAllQuotes(0, 10)).withRel("quotes").getHref();
        String actualHref = model.getRequiredLink("quotes").getHref();
        assertEquals(expectedHref, actualHref);
    }
}
