package com.example.wfwebapi.controller;

import com.example.wfwebapi.exception.ResourceNotFoundException;
import com.example.wfwebapi.model.Admin;
import com.example.wfwebapi.repository.AdminRepository;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.web.server.LocalServerPort;
import org.springframework.http.ResponseEntity;
import org.springframework.web.client.RestTemplate;

import static org.junit.jupiter.api.Assertions.*;

@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
class AdminControllerTest {

    @LocalServerPort
    private int port;

    @Autowired
    private AdminRepository adminRepository;

    private String baseUrl() {
        return "http://localhost:" + port + "/api/v1/admin/";
    }

    @Test
    void testGetAdminFound() {
        Admin admin = new Admin();
        admin.setUserId(999L);
        adminRepository.save(admin);

        RestTemplate restTemplate = new RestTemplate();
        ResponseEntity<Admin> response = restTemplate.getForEntity(baseUrl() + "999", Admin.class);

        assertEquals(200, response.getStatusCodeValue());
        assertEquals(999L, response.getBody().getUserId());
    }

    @Test
    void testGetAdminNotFound() {
        RestTemplate restTemplate = new RestTemplate();

        Exception exception = assertThrows(Exception.class, () ->
                restTemplate.getForEntity(baseUrl() + "123456", Admin.class)
        );

        assertTrue(exception.getMessage().contains("404"));
    }
}
