package com.example.wfwebapi.controller;

import com.example.wfwebapi.model.*;
import com.example.wfwebapi.repository.*;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.boot.test.web.server.LocalServerPort;
import org.springframework.http.*;

import java.time.LocalDate;

import static org.junit.jupiter.api.Assertions.*;

@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
class DailyQuoteControllerTest {

    @LocalServerPort
    private int port;

    @Autowired private TestRestTemplate restTemplate;

    @Autowired private DailyQuoteRepository dailyQuoteRepository;
    @Autowired private QuoteRepository quoteRepository;
    @Autowired private UserRepository userRepository;
    @Autowired private ImageRepository imageRepository;
    @Autowired private CaracterRepository caracterRepository;
    @Autowired private SourceRepository sourceRepository;

    private Quote quote;

    private String getUrl() {
        return "http://localhost:" + port + "/api/v1/dailyquote";
    }

    @BeforeEach
    void setup() {
        dailyQuoteRepository.deleteAll();
        quoteRepository.deleteAll();
        caracterRepository.deleteAll();
        sourceRepository.deleteAll();
        userRepository.deleteAll();
        imageRepository.deleteAll();

        Image image = new Image();
        image.setImgPath("daily.png");
        imageRepository.save(image);

        User user = new User();
        user.setUsername("dailyUser");
        user.setPassword("pwd");
        user.setEmail("daily@mail.com");
        user.setCreation(LocalDate.now());
        user.setImage(image);
        userRepository.save(user);

        Caracter caracter = new Caracter();
        caracter.setCaracter("Calme");
        caracter.setImage(image);
        caracterRepository.save(caracter);

        Source source = new Source();
        source.setTitle("Livre A");
        source.setDateS(2001);
        sourceRepository.save(source);

        quote = new Quote();
        quote.setContent("Chaque jour est un cadeau.");
        quote.setLangue("fr");
        quote.setIsValide(true);
        quote.setLikes(10);
        quote.setReason("approuvée");
        quote.setUserVerif(user);
        quote.setCaracter(caracter);
        quote.setSource(source);
        quoteRepository.save(quote);

        DailyQuote dailyQuote = new DailyQuote();
        dailyQuote.setCitationId(quote.getId()); // CORRECT : ne pas faire setQuote()
        dailyQuoteRepository.save(dailyQuote);
    }

    @Test
    void testGetDailyQuote() {
        ResponseEntity<DailyQuote> response = restTemplate.getForEntity(getUrl(), DailyQuote.class);

        assertEquals(HttpStatus.OK, response.getStatusCode());
        assertNotNull(response.getBody());
        assertEquals(quote.getId(), response.getBody().getCitationId());
    }

    @Test
    void testGetDailyQuoteNotFound() {
        dailyQuoteRepository.deleteAll();

        ResponseEntity<String> response = restTemplate.getForEntity(getUrl(), String.class);

        assertEquals(HttpStatus.NOT_FOUND, response.getStatusCode());
        assertTrue(response.getBody().contains("Aucune DailyQuote définie"));
    }
}
