package com.example.wfwebapi.controller;

import com.example.wfwebapi.model.Question;
import com.example.wfwebapi.repository.QuestionRepository;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.boot.test.web.server.LocalServerPort;
import org.springframework.http.*;

import static org.junit.jupiter.api.Assertions.*;

@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
class QuestionControllerTest {

    @LocalServerPort
    private int port;

    @Autowired private TestRestTemplate restTemplate;
    @Autowired private QuestionRepository questionRepository;

    private String getUrl(String path) {
        return "http://localhost:" + port + "/api/v1/question" + path;
    }

    @BeforeEach
    void setup() {
        questionRepository.deleteAll();
    }

    @Test
    void testCreateAndGetQuestion() {
        Question question = new Question();
        question.setTexte("Quel est le langage de ce test ?");
        question.setAnswerA("Python");
        question.setAnswerB("Java");
        question.setAnswerC("C++");
        question.setAnswerD("JavaScript");
        question.setCAnswer("Java");

        ResponseEntity<Question> postResponse = restTemplate.postForEntity(getUrl(""), question, Question.class);
        assertEquals(HttpStatus.OK, postResponse.getStatusCode());
        assertNotNull(postResponse.getBody());
        Long id = postResponse.getBody().getId();
        assertNotNull(id);

        ResponseEntity<Question> getResponse = restTemplate.getForEntity(getUrl("/" + id), Question.class);
        assertEquals(HttpStatus.OK, getResponse.getStatusCode());
        assertEquals("Java", getResponse.getBody().getCAnswer());
    }

    @Test
    void testUpdateQuestion() {
        Question q = new Question();
        q.setTexte("Old Q");
        q.setAnswerA("A");
        q.setAnswerB("B");
        q.setAnswerC("C");
        q.setAnswerD("D");
        q.setCAnswer("B");
        q = questionRepository.save(q);

        q.setTexte("Updated Q");
        q.setCAnswer("A");

        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        HttpEntity<Question> entity = new HttpEntity<>(q, headers);

        ResponseEntity<Question> response = restTemplate.exchange(
                getUrl(""), HttpMethod.PUT, entity, Question.class);

        assertEquals(HttpStatus.OK, response.getStatusCode());
        assertEquals("Updated Q", response.getBody().getTexte());
        assertEquals("A", response.getBody().getCAnswer());
    }

    @Test
    void testDeleteQuestion() {
        Question q = new Question();
        q.setTexte("À supprimer");
        q.setAnswerA("1");
        q.setAnswerB("2");
        q.setAnswerC("3");
        q.setAnswerD("4");
        q.setCAnswer("2");
        q = questionRepository.save(q);

        ResponseEntity<Void> delete = restTemplate.exchange(
                getUrl("?id=" + q.getId()), HttpMethod.DELETE, null, Void.class);

        assertEquals(HttpStatus.OK, delete.getStatusCode());
        assertFalse(questionRepository.existsById(q.getId()));
    }

    @Test
    void testGetQuestionNotFound() {
        ResponseEntity<String> response = restTemplate.getForEntity(getUrl("/9999"), String.class);
        assertEquals(HttpStatus.NOT_FOUND, response.getStatusCode());
        assertTrue(response.getBody().contains("Question non trouvée"));
    }

    @Test
    void testDeleteQuestionNotFound() {
        ResponseEntity<String> response = restTemplate.exchange(
                getUrl("?id=9999"), HttpMethod.DELETE, null, String.class);
        assertEquals(HttpStatus.NOT_FOUND, response.getStatusCode());
        assertTrue(response.getBody().contains("Question non trouvée"));
    }
}
